#include <stdio.h>
#include <queue>
#include <vector>
using namespace std;

#define MAX 1024
#define INF 999666111

struct node { int dest, cost, len; node(){} node(int _dest, int _cost): dest(_dest), len(_cost){} };

bool use_heap = true, use_bin = true;
int g[MAX][MAX], xg[MAX][MAX];
vector<node> neigh[MAX];
int ds[MAX];
int ps[MAX];
int n, m, c, p;
int ans;


void handle_args(int argc, char **argv)
{
	for (int i = 1; i < argc; i++) {
		if (!strcmp(argv[i], "-h")) {
			printf("Add --heap or --no-heap to enable/disable heaped Dijkstra\n");
			printf("Add --bin or --no-bin to enable/disable binary search\n");
		}
		if (!strcmp(argv[i], "--heap")) use_heap = true;
		if (!strcmp(argv[i], "--no-heap")) use_heap = false;
		if (!strcmp(argv[i], "--bin")) use_bin = true;
		if (!strcmp(argv[i], "--no-bin")) use_bin = false;
	}
}

void reset(void)
{
	for (int i = 0; i < MAX; i++) for (int j = 0; j < MAX; j++) g[i][j] = xg[i][j] = INF;
	ans = INF;
	for (int i = 0; i < MAX; i++) neigh[i].clear();
}

void input(void)
{
	scanf("%d%d%d%d", &n, &m, &c, &p);
	for (int i = 0; i < m; i++) {
		int x, y, z;
		scanf("%d%d%d", &x, &y, &z);
		x--,y--;
		if (z < xg[x][y]) {
			xg[x][y] = z;
			unsigned j = 0;
			while (j < neigh[x].size() && neigh[x][j].dest != y) j++;
			if (j < neigh[x].size() && neigh[x][j].dest == y) {
				neigh[x][j].len = z;
			} else {
				neigh[x].push_back(node(y, z));
			}
		}
	}
}

void dijkstra(int root)
{
	for (int i = 0; i < n; i++) {
		ds[i] = INF;
		ps[i] = 0;
	}
	ds[root] = 0;
	while (1) {
		int bi = -1, mdist = INF;
		for (int i = 0; i < n; i++) if (!ps[i] && ds[i] < mdist) {
			mdist = ds[i];
			bi = i;
		}
		if (-1 == bi) return;
		ps[bi] = 1;
		for (int i = 0; i < n; i++) if (!ps[i] && ds[i] > ds[bi] + g[bi][i]) {
			ds[i] = ds[bi] + g[bi][i];
		}
	}
}

void heaped_dijkstra(int root)
{
	for (int i = 0; i < n; i++) {
		ds[i] = INF;
		ps[i] = 0;
	}
	ds[root] = 0;
	priority_queue<pair<int, int> > q;
	q.push(make_pair(0, root));
	while (1) {
		int bi;
		while (!q.empty() && ps[q.top().second])
			q.pop();
		if (q.empty()) return;
		bi = q.top().second;
		q.pop();
		ps[bi] = 1;
		for (unsigned ii = 0; ii < neigh[bi].size(); ii++) {
 			int i = neigh[bi][ii].dest;
 			int x = neigh[bi][ii].cost;
 			if (ds[i] > ds[bi] + x) {
 				ds[i] = ds[bi] + x;
 				q.push(make_pair(-ds[i], i));
 			}
		}
	}
}

int evaluate(int k)
{
	for (int i = 0; i < n; i++) {
		for (int j = 0; j < n; j++) {
			if (xg[i][j] < INF) g[i][j] = max(0, (xg[i][j]-k) * c);
		}
	}
	for (int i = 0; i < n; i++)
		for (unsigned j = 0; j < neigh[i].size(); j++)
			neigh[i][j].cost = max(0, (neigh[i][j].len-k)*c);
	if (use_heap)
		heaped_dijkstra(0);
	else
		dijkstra(0);
	int ans = ds[n-1] + k * p;
	return ans;
}

void solve(void)
{
	if (use_bin) {
		int l = 0, r = 1000;
		while (l != r) {
			int m = (l+r)/2;
			int t1 = evaluate(m);
			int t2 = evaluate(m+1);
			if (t1 == t2) t2 = evaluate(m + 2);
			if (t1 < t2) {
				r = m;
			} else {
				l = m + 1;
			}
			ans = min(ans, min(t1, t2));
		}
		ans = min(ans, evaluate(l));
	} else {
		for (int p = 0; p <= 1000; p++) {
			ans = min(ans, evaluate(p));
		}
	}
}

int main(int argc, char **argv)
{
	handle_args(argc, argv);
	//
	int T;
	scanf("%d", &T);
	while (T--) {
		reset();
		input();
		solve();
		printf("%d\n", ans);
	}
	return 0;
}
